<?php

/**
 * @file
 * Contains \Drupal\datetime\Tests\Views\ArgumentDefaultDateTimeTest.
 */

namespace Drupal\datetime\Tests\Views;

use Drupal\views\Views;

/**
 * Tests the Drupal\datetime\Plugin\views\filter\Date handler.
 *
 * @group datetime
 */
class ArgumentDefaultDateTimeTest extends DateTimeHandlerTestBase {

  /**
   * {@inheritdoc}
   */
  public static $testViews = ['test_argument_default_datetime'];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    /** @var \Drupal\Core\Datetime\DateFormatterInterface $date_formatter */
    $date_formatter = \Drupal::service('date.formatter');
    $request_time = \Drupal::requestStack()->getCurrentRequest()->server->get('REQUEST_TIME');
    $year = (int) $date_formatter->format($request_time, 'custom', 'Y');
    $yearp = $year - 1;
    $yearf = $year + 1;

    // Add some basic test nodes.
    $dates = [
      $yearp . '-10-10',
      $year . '-10-10',
      $yearf . '-10-10',
    ];
    foreach ($dates as $date) {
      $this->nodes[] = $this->drupalCreateNode([
        'field_date' => [
          'value' => $date,
        ]
      ]);
    }
  }

  /**
   * Test year argument.
   *
   * @see \Drupal\datetime\Plugin\views\argument\YearDate
   */
  public function testDatetimeArgumentDefaultYear() {
    $view = Views::getView('test_argument_default_datetime');

    // The 'default' display has the 'year' argument, with the default coming
    // from the current date.
    $view->setDisplay('default');
    $this->executeView($view, []);
    $expected = [];
    $expected[] = ['nid' => $this->nodes[1]->id()];
    $this->assertIdenticalResultset($view, $expected, $this->map);
    $view->destroy();
  }

}
